"use server";

import { validateRequest } from "@/server/auth";
import { UserRole } from "@/server/auth/permissions";
import { db } from "@/server/db";
import { printJobs, users } from "@/server/db/schema";
import { IS, guard } from "@/utils/guard";
import strings from "@/utils/strings";
import { type InferInsertModel, eq } from "drizzle-orm";
import { revalidatePath } from "next/cache";

export async function createPrintJob(printJob: InferInsertModel<typeof printJobs>) {
	const { user } = await validateRequest();

	if (guard(user, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Bereite die Daten für das Backend vor
		// In unserem Backend wird printerId als socketId bezeichnet
		const backendJob = {
			socketId: printJob.printerId,  // Feldname ändern für Backend
			userId: printJob.userId,
			durationInMinutes: printJob.durationInMinutes,
			comments: printJob.comments || ""
		};
		
		// Sende den Job ans Backend
		const backendResponse = await fetch(`${process.env.BACKEND_URL || 'http://localhost:5000'}/api/jobs`, {
			method: 'POST',
			headers: {
				'Content-Type': 'application/json'
			},
			body: JSON.stringify(backendJob)
		});
		
		if (!backendResponse.ok) {
			throw new Error(`Backend-Fehler: ${backendResponse.status} ${backendResponse.statusText}`);
		}
		
		const backendResult = await backendResponse.json();
		
		// Synchronisiere mit lokaler Datenbank
		// Wir verwenden hier die Job-ID vom Backend, um Konsistenz zu gewährleisten
		if (backendResult.id) {
			const result = await db.insert(printJobs).values({
				...printJob,
				id: backendResult.id // Verwende die vom Backend erzeugte ID
			}).returning({
				jobId: printJobs.id,
			});
			return result[0].jobId;
		} else {
			return backendResult.id;
		}
	} catch (error) {
		console.error("Fehler beim Erstellen des Druckauftrags:", error);
		return {
			error: error instanceof Error ? error.message : "Druckauftrag konnte nicht hinzugefügt werden.",
		};
	}
}

/* async function updatePrintJob(jobId: string, printJob: InferInsertModel<typeof printJobs>) {
	const { user } = await validateRequest();

	if (guard(user, is, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION
		}
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, is, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION
		}
	}

	await db.update(printJobs).set(printJob).where(eq(printJobs.id, jobId));
} */

export async function abortPrintJob(jobId: string, reason: string) {
	const { user } = await validateRequest();

	if (guard(user, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Hole den Druckauftrag vom Backend
		const backendResponse = await fetch(`${process.env.BACKEND_URL || 'http://localhost:5000'}/api/jobs/${jobId}/abort`, {
			method: 'POST',
			headers: {
				'Content-Type': 'application/json'
			},
			body: JSON.stringify({ reason })
		});
		
		if (!backendResponse.ok) {
			const errorData = await backendResponse.json();
			throw new Error(errorData.message || `Backend-Fehler: ${backendResponse.status}`);
		}
		
		// Hole den lokalen Druckauftrag für die Berechtigungsprüfung
		const printJob = await db.query.printJobs.findFirst({
			where: eq(printJobs.id, jobId),
		});

		if (printJob) {
			// Aktualisiere den lokalen Druckauftrag auch
			// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
			await db
				.update(printJobs)
				.set({
					aborted: true,
					abortReason: reason,
					comments: `${printJob.comments || ""}\n\n---${dbUser?.username}: Druckauftrag abgebrochen`,
				})
				.where(eq(printJobs.id, jobId));
		}
		
		revalidatePath("/");
		return { success: true };
	} catch (error) {
		console.error("Fehler beim Abbrechen des Druckauftrags:", error);
		return {
			error: error instanceof Error ? error.message : "Druckauftrag konnte nicht abgebrochen werden.",
		};
	}
}

export async function earlyFinishPrintJob(jobId: string) {
	const { user } = await validateRequest();

	if (guard(user, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Sende die Anfrage an das Backend
		const backendResponse = await fetch(`${process.env.BACKEND_URL || 'http://localhost:5000'}/api/jobs/${jobId}/finish`, {
			method: 'POST',
			headers: {
				'Content-Type': 'application/json'
			}
		});
		
		if (!backendResponse.ok) {
			const errorData = await backendResponse.json();
			throw new Error(errorData.message || `Backend-Fehler: ${backendResponse.status}`);
		}
		
		// Hole den lokalen Druckauftrag für die Synchronisierung
		const printJob = await db.query.printJobs.findFirst({
			where: eq(printJobs.id, jobId),
		});

		if (printJob) {
			// Aktualisiere den lokalen Druckauftrag auch
			// Hole die tatsächliche Dauer vom Backend
			const backendData = await backendResponse.json();
			const duration = backendData.durationInMinutes || Math.floor((Date.now() - new Date(printJob.startAt).getTime()) / 1000 / 60);
			
			await db
				.update(printJobs)
				.set({
					durationInMinutes: duration,
					comments: `${printJob.comments || ""}\n\n---${dbUser?.username}: Druckauftrag vorzeitig abgeschlossen`,
				})
				.where(eq(printJobs.id, jobId));
		}
		
		revalidatePath("/");
		return { success: true };
	} catch (error) {
		console.error("Fehler beim vorzeitigen Beenden des Druckauftrags:", error);
		return {
			error: error instanceof Error ? error.message : "Druckauftrag konnte nicht vorzeitig beendet werden.",
		};
	}
}

export async function extendPrintJob(jobId: string, minutes: number, hours: number) {
	const { user } = await validateRequest();

	if (guard(user, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Sende die Anfrage an das Backend
		const backendResponse = await fetch(`${process.env.BACKEND_URL || 'http://localhost:5000'}/api/jobs/${jobId}/extend`, {
			method: 'POST',
			headers: {
				'Content-Type': 'application/json'
			},
			body: JSON.stringify({ minutes, hours })
		});
		
		if (!backendResponse.ok) {
			const errorData = await backendResponse.json();
			throw new Error(errorData.message || `Backend-Fehler: ${backendResponse.status}`);
		}
		
		// Hole den lokalen Druckauftrag für die Synchronisierung
		const printJob = await db.query.printJobs.findFirst({
			where: eq(printJobs.id, jobId),
		});

		if (printJob) {
			const duration = minutes + hours * 60;
			
			// Aktualisiere den lokalen Druckauftrag auch
			await db
				.update(printJobs)
				.set({
					durationInMinutes: printJob.durationInMinutes + duration,
					comments: `${printJob.comments || ""}\n\n---${dbUser?.username}: Verlängert um ${hours} Stunden und ${minutes} Minuten`,
				})
				.where(eq(printJobs.id, jobId));
		}
		
		revalidatePath("/");
		return { success: true };
	} catch (error) {
		console.error("Fehler beim Verlängern des Druckauftrags:", error);
		return {
			error: error instanceof Error ? error.message : "Druckauftrag konnte nicht verlängert werden.",
		};
	}
}

export async function updatePrintComments(jobId: string, comments: string) {
	const { user } = await validateRequest();

	if (guard(user, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS, UserRole.GUEST)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Sende die Anfrage an das Backend
		const backendResponse = await fetch(`${process.env.BACKEND_URL || 'http://localhost:5000'}/api/jobs/${jobId}/comments`, {
			method: 'PUT',
			headers: {
				'Content-Type': 'application/json'
			},
			body: JSON.stringify({ comments })
		});
		
		if (!backendResponse.ok) {
			const errorData = await backendResponse.json();
			throw new Error(errorData.message || `Backend-Fehler: ${backendResponse.status}`);
		}

		// Synchronisiere mit der lokalen Datenbank
		const printJob = await db.query.printJobs.findFirst({
			where: eq(printJobs.id, jobId),
		});

		if (printJob) {
			await db
				.update(printJobs)
				.set({
					comments,
				})
				.where(eq(printJobs.id, jobId));
		}
		
		revalidatePath("/");
		return { success: true };
	} catch (error) {
		console.error("Fehler beim Aktualisieren der Kommentare:", error);
		return {
			error: error instanceof Error ? error.message : "Kommentare konnten nicht aktualisiert werden.",
		};
	}
}
