"use server";
import { validateRequest } from "@/server/auth";
import { UserRole } from "@/server/auth/permissions";
import { db } from "@/server/db";
import { printers, users } from "@/server/db/schema";
import { IS_NOT, guard } from "@/utils/guard";
import strings from "@/utils/strings";
import { type InferInsertModel, eq } from "drizzle-orm";
import { revalidatePath } from "next/cache";
import { backendApi } from "@/utils/fetch";
import { mapBackendPrinterToFrontend, mapFrontendPrinterToBackend } from "@/utils/backend-types";

export async function createPrinter(printer: InferInsertModel<typeof printers>) {
	const { user } = await validateRequest();

	if (guard(user, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	if (!printer) {
		return {
			error: "Druckerdaten sind erforderlich.",
		};
	}

	try {
		// Transformiere die Daten ins Backend-Format
		const backendPrinter = mapFrontendPrinterToBackend(printer);
		
		// Rufe das Backend API auf
		await backendApi.printers.create(backendPrinter);
		
		// Lokale Datenbank bleibt synchronisiert (optional, wenn du weiterhin lokale Daten brauchst)
		await db.insert(printers).values(printer);
	} catch (error) {
		console.error("Fehler beim Erstellen des Druckers:", error);
		return {
			error: "Drucker konnte nicht hinzugefügt werden.",
		};
	}

	revalidatePath("/");
}

export async function updatePrinter(id: string, data: InferInsertModel<typeof printers>) {
	const { user } = await validateRequest();

	if (guard(user, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	if (!data) {
		return {
			error: "Druckerdaten sind erforderlich.",
		};
	}

	try {
		// Transformiere die Daten ins Backend-Format
		const backendPrinter = mapFrontendPrinterToBackend(data);
		
		// Rufe das Backend API auf
		await backendApi.printers.update(id, backendPrinter);
		
		// Lokale Datenbank bleibt synchronisiert (optional)
		await db.update(printers).set(data).where(eq(printers.id, id));
	} catch (error) {
		console.error("Fehler beim Aktualisieren des Druckers:", error);
		return {
			error: "Drucker konnte nicht aktualisiert werden.",
		};
	}

	revalidatePath("/");
}

export async function deletePrinter(id: string) {
	const { user } = await validateRequest();

	if (guard(user, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	const dbUser = await db.query.users.findFirst({
		// biome-ignore lint/style/noNonNullAssertion: guard already checks against null
		where: eq(users.id, user!.id),
	});

	if (guard(dbUser, IS_NOT, UserRole.ADMIN)) {
		return {
			error: strings.ERROR.PERMISSION,
		};
	}

	try {
		// Rufe das Backend API auf
		await backendApi.printers.delete(id);
		
		// Lokale Datenbank bleibt synchronisiert (optional)
		await db.delete(printers).where(eq(printers.id, id));
	} catch (error) {
		console.error("Fehler beim Löschen des Druckers:", error);
		if (error instanceof Error) {
			return {
				error: error.message,
			};
		}
		return {
			error: "Ein unbekannter Fehler ist aufgetreten.",
		};
	}

	revalidatePath("/");
}

export async function getPrinters() {
	try {
		// Rufe die Drucker vom Backend ab
		const backendPrinters = await backendApi.printers.getAll();
		
		// Transformiere die Backend-Daten ins Frontend-Format
		return backendPrinters.map(mapBackendPrinterToFrontend);
	} catch (error) {
		console.error("Fehler beim Abrufen der Drucker:", error);
		
		// Fallback zur lokalen Datenbank, falls das Backend nicht erreichbar ist
		return await db.query.printers.findMany({
			with: {
				printJobs: {
					limit: 1,
					orderBy: (printJobs, { desc }) => [desc(printJobs.startAt)],
				},
			},
		});
	}
}
